# -*- coding: utf-8 -*-
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _

class MyTestApphook(CMSApp):
    name = _("My Apphook")
    urls = ["aldryn_apphook_reload.test_utils.test_app.urls"]

apphook_pool.register(MyTestApphook)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import sys
import uuid
from django.conf import settings
import django.core.urlresolvers
from django.core.urlresolvers import reverse
import cms.appresolver
from threading import local
import cms.apphook_pool


_urlconf_revision = {}
_urlconf_revision_threadlocal = local()

use_threadlocal = False


def ensure_urlconf_is_up_to_date():
    global_revision = get_global_revision()
    local_revision = get_local_revision()
    if global_revision != local_revision:
        print "    new revision!!!! RELOAD!\n      {} ({})\n   -> {} ({})".format(
            global_revision, type(global_revision),
            local_revision, type(local_revision),
        )
        debug_check_url('my_test_app_view')
        reload_urlconf(new_revision=global_revision)
        debug_check_url('my_test_app_view')


def get_local_revision(default=None):
    if use_threadlocal:
        return getattr(_urlconf_revision_threadlocal, "value", default)
    else:
        global _urlconf_revision
        return _urlconf_revision.get('urlconf_revision', default)


def set_local_revision(revision):
    # print '======= SETTING =====', revision
    if use_threadlocal:
        if revision:
            _urlconf_revision_threadlocal.value = revision
            print '======= SET =====    ', get_local_revision()
        else:
            print '======= DEL =====    ', revision
            if hasattr(_urlconf_revision_threadlocal, "value"):
                del _urlconf_revision_threadlocal.value
    else:
        global _urlconf_revision
        _urlconf_revision['urlconf_revision'] = revision


def get_global_revision():
    from .models import UrlconfRevision
    revision, created = UrlconfRevision.objects.get_or_create(
        id=1,
        defaults=dict(
            revision=str(uuid.uuid4()),
        )
    )
    return revision.revision


def set_global_revision(new_revision=None):
    from .models import UrlconfRevision
    x = UrlconfRevision.objects.update(id=1, revision=new_revision)
    if x < 1:
        # the revision entry in the db does not exist yet
        UrlconfRevision.objects.get_or_create(
            id=1,
            defaults=dict(
                revision=new_revision,
            )
        )


def mark_urlconf_as_changed():
    new_revision = str(uuid.uuid4())
    set_global_revision(new_revision=new_revision)
    return new_revision


def reload_urlconf(urlconf=None, new_revision=None):
    if 'cms.urls' in sys.modules:
        reload(sys.modules['cms.urls'])
    if urlconf is None:
        urlconf = settings.ROOT_URLCONF
    if urlconf in sys.modules:
        reload(sys.modules[urlconf])
    cms.appresolver.clear_app_resolvers()
    django.core.urlresolvers.clear_url_caches()
    cms.appresolver.get_app_patterns()
    if new_revision is not None:
        set_local_revision(new_revision)


def debug_check_url(url_name):
    # try:
    #     print """    django.core.urlresolvers.reverse('{}'): {} """.format(
    #         url_name,
    #         django.core.urlresolvers.reverse(url_name)
    #     )
    # except Exception as e:
    #     print "django.core.urlresolvers.reverse('{}'): {}".format(
    #         url_name,
    #         e,
    #     )
    try:
        print """    reverse('{}'): {} """.format(
            url_name,
            reverse('my_test_app_view'),
        )
    except Exception as e:
        print "reverse('{}'): {}".format(
            url_name,
            e,
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from .models import UrlconfRevision
from django.contrib import admin


class UrlconfAdmin(admin.ModelAdmin):
    list_display = (
        'revision',
    )


admin.site.register(UrlconfRevision, UrlconfAdmin)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import cms.signals
from . import utils


def trigger_server_restart(**kwargs):
    utils.mark_urlconf_as_changed()


cms.signals.urls_need_reloading.connect(
    trigger_server_restart, dispatch_uid='aldryn-apphook-reload-handle-urls-need-reloading')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from . import utils


class ApphookReloadMiddleware(object):
    """
    redirects any alias domains to the main domain.
    Takes into account if SECURE_SSL_REDIRECT (from django-secure) is set and redirects directly to the
    https version of the main domain if that is true.

    This middleware must be BEFORE djangosecure.middleware.SecurityMiddleware in MIDDLEWARE_CLASSES, so it can
    prevent the redirect from an alias domain to it's https version by djangosecure.middleware.SecurityMiddleware,
    because you might only cover the main domain with the certificate.
    """
    def process_request(self, request):
        utils.ensure_urlconf_is_up_to_date()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.db import models

from . import signals  # needed for signal registration


class UrlconfRevision(models.Model):
    revision = models.CharField(max_length=255)



# -*- coding: utf-8 -*-
from django.conf.urls import *
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.views.generic import View


class MyTestView(View):
    def get(self, request, *args, **kwargs):
        try:
            url = reverse('my_test_app_view')
        except Exception as e:
            url = 'failed: {}'.format(e)
        # return HttpResponse("My Apphook Page (resolving to {})".format(url))
        return HttpResponse("My Apphook Page")


urlpatterns = patterns('',
    url(r'^$', MyTestView.as_view(), name='my_test_app_view'),
)



# -*- coding: utf-8 -*-
from django.conf.urls import *
from django.core.urlresolvers import reverse
import django.core.urlresolvers
from django.http import HttpResponse
from django.views.generic import View


class MyTestView(View):
    def get(self, request, *args, **kwargs):
        try:
            url1 = reverse('my_test_app_view')
            url2 = django.core.urlresolvers.reverse('my_test_app_view')
            if url1 == url2:
                url = url1
            else:
                raise Exception('urls not the same!!!!')
        except Exception as e:
            url = 'failed: {}'.format(e)
        return HttpResponse("Apphook Test: {}".format(url))


urlpatterns = patterns('',
    url(r'^$', MyTestView.as_view(), name='my_test_app_view_test'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.apps import AppConfig


class AldrynApphookReloadConfig(AppConfig):
    name = 'aldryn_apphook_reload'
    verbose_name = "Aldryn Apphook Reload"
